@@ -11,7 +11,7 @@ from django.template.defaultfilters import filesizeformat
 import sentry_sdk
 
 
-from swh.model.identifiers import DIRECTORY
+from swh.model.identifiers import DIRECTORY, RELEASE, REVISION, SNAPSHOT
 from swh.web.browse.browseurls import browse_route
 from swh.web.browse.snapshot_context import get_snapshot_context
 from swh.web.browse.utils import (
@@ -23,7 +23,7 @@ from swh.web.common import service
 from swh.web.common.exc import handle_view_exception, NotFoundExc
 from swh.web.common.identifiers import get_swhids_info
 from swh.web.common.typing import DirectoryMetadata, SWHObjectInfo
-from swh.web.common.utils import reverse, gen_path_info
+from swh.web.common.utils import reverse, gen_path_info, swh_object_icons
 
 
 def _directory_browse(request, sha1_git, path=None):
@@ -34,36 +34,44 @@ def _directory_browse(request, sha1_git, path=None):
             sha1_git = dir_info["target"]
 
         dirs, files = get_directory_entries(sha1_git)
-        origin_url = request.GET.get("origin_url", None)
+        origin_url = request.GET.get("origin_url")
         if not origin_url:
-            origin_url = request.GET.get("origin", None)
+            origin_url = request.GET.get("origin")
+        snapshot_id = request.GET.get("snapshot")
         snapshot_context = None
-        if origin_url:
+        if origin_url is not None or snapshot_id is not None:
             try:
-                snapshot_context = get_snapshot_context(origin_url=origin_url)
-            except NotFoundExc:
-                raw_dir_url = reverse(
-                    "browse-directory", url_args={"sha1_git": sha1_git}
+                snapshot_context = get_snapshot_context(
+                    snapshot_id=snapshot_id,
+                    origin_url=origin_url,
+                    branch_name=request.GET.get("branch"),
+                    release_name=request.GET.get("release"),
+                    revision_id=request.GET.get("revision"),
+                    path=path,
                 )
-                error_message = (
-                    "The Software Heritage archive has a directory "
-                    "with the hash you provided but the origin "
-                    "mentioned in your request appears broken: %s. "
-                    "Please check the URL and try again.\n\n"
-                    "Nevertheless, you can still browse the directory "
-                    "without origin information: %s"
-                    % (gen_link(origin_url), gen_link(raw_dir_url))
-                )
-
-                raise NotFoundExc(error_message)
-        if snapshot_context:
-            snapshot_context["visit_info"] = None
+            except NotFoundExc as e:
+                if str(e).startswith("Origin"):
+                    raw_dir_url = reverse(
+                        "browse-directory", url_args={"sha1_git": sha1_git}
+                    )
+                    error_message = (
+                        "The Software Heritage archive has a directory "
+                        "with the hash you provided but the origin "
+                        "mentioned in your request appears broken: %s. "
+                        "Please check the URL and try again.\n\n"
+                        "Nevertheless, you can still browse the directory "
+                        "without origin information: %s"
+                        % (gen_link(origin_url), gen_link(raw_dir_url))
+                    )
+                    raise NotFoundExc(error_message)
+                else:
+                    raise e
     except Exception as exc:
         return handle_view_exception(request, exc)
 
     path_info = gen_path_info(path)
 
-    query_params = {"origin_url": origin_url}
+    query_params = snapshot_context["query_params"] if snapshot_context else {}
 
     breadcrumbs = []
     breadcrumbs.append(
@@ -115,7 +123,7 @@ def _directory_browse(request, sha1_git, path=None):
             url_args={"query_string": query_string},
             query_params={
                 "path": root_sha1_git + "/" + path + f["name"],
-                "origin_url": origin_url,
+                **query_params,
             },
         )
         if f["length"] is not None:
@@ -152,6 +160,24 @@ def _directory_browse(request, sha1_git, path=None):
 
     swh_objects = [SWHObjectInfo(object_type=DIRECTORY, object_id=sha1_git)]
 
+    if snapshot_context:
+        swh_objects.append(
+            SWHObjectInfo(
+                object_type=REVISION, object_id=snapshot_context["revision_id"]
+            )
+        )
+        swh_objects.append(
+            SWHObjectInfo(
+                object_type=SNAPSHOT, object_id=snapshot_context["snapshot_id"]
+            )
+        )
+        if snapshot_context["release_id"]:
+            swh_objects.append(
+                SWHObjectInfo(
+                    object_type=RELEASE, object_id=snapshot_context["release_id"]
+                )
+            )
+
     swhids_info = get_swhids_info(swh_objects, snapshot_context, dir_metadata)
 
     heading = "Directory - %s" % sha1_git
@@ -159,6 +185,19 @@ def _directory_browse(request, sha1_git, path=None):
         dir_path = "/".join([bc["name"] for bc in breadcrumbs]) + "/"
         heading += " - %s" % dir_path
 
+    top_right_link = None
+    if snapshot_context is not None and not snapshot_context["is_empty"]:
+        history_url = reverse(
+            "browse-revision-log",
+            url_args={"sha1_git": snapshot_context["revision_id"]},
+            query_params=query_params,
+        )
+        top_right_link = {
+            "url": history_url,
+            "icon": swh_object_icons["revisions history"],
+            "text": "History",
+        }
+
     return render(
         request,
         "browse/directory.html",
@@ -170,7 +209,7 @@ def _directory_browse(request, sha1_git, path=None):
             "dirs": dirs,
             "files": files,
             "breadcrumbs": breadcrumbs,
-            "top_right_link": None,
+            "top_right_link": top_right_link,
             "readme_name": readme_name,
             "readme_url": readme_url,
             "readme_html": readme_html,
