@@ -1,4 +1,4 @@
-# Copyright (C) 2015-2019  The Software Heritage developers
+# Copyright (C) 2015-2020  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU Affero General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -7,9 +7,10 @@ from hypothesis import given
 import pytest
 from requests.utils import parse_header_links
 
-from swh.model.model import Origin
+from swh.model.model import Origin, OriginVisit, OriginVisitStatus
 
 from swh.storage.exc import StorageDBError, StorageAPIError
+from swh.storage.utils import now
 
 from swh.web.api.utils import enrich_origin_visit, enrich_origin
 from swh.web.common.exc import BadInputExc
@@ -101,15 +102,25 @@ def test_api_lookup_origin_visits(
     archive_data.origin_add_one(new_origin)
     for i, visit_date in enumerate(visit_dates):
         origin_visit = archive_data.origin_visit_add(
-            new_origin.url, visit_date, type="git"
-        )
+            [
+                OriginVisit(
+                    origin=new_origin.url,
+                    date=visit_date,
+                    type="git",
+                    status="ongoing",
+                    snapshot=None,
+                )
+            ]
+        )[0]
         archive_data.snapshot_add([new_snapshots[i]])
-        archive_data.origin_visit_update(
-            new_origin.url,
-            origin_visit.visit,
+        visit_status = OriginVisitStatus(
+            origin=new_origin.url,
+            visit=origin_visit.visit,
+            date=now(),
             status="full",
             snapshot=new_snapshots[i].id,
         )
+        archive_data.origin_visit_status_add([visit_status])
 
     all_visits = list(reversed(get_origin_visits(new_origin.to_dict())))
 
@@ -147,15 +158,25 @@ def test_api_lookup_origin_visits_by_id(
     archive_data.origin_add_one(new_origin)
     for i, visit_date in enumerate(visit_dates):
         origin_visit = archive_data.origin_visit_add(
-            new_origin.url, visit_date, type="git"
-        )
+            [
+                OriginVisit(
+                    origin=new_origin.url,
+                    date=visit_date,
+                    type="git",
+                    status="ongoing",
+                    snapshot=None,
+                )
+            ]
+        )[0]
         archive_data.snapshot_add([new_snapshots[i]])
-        archive_data.origin_visit_update(
-            new_origin.url,
-            origin_visit.visit,
+        visit_status = OriginVisitStatus(
+            origin=new_origin.url,
+            visit=origin_visit.visit,
+            date=now(),
             status="full",
             snapshot=new_snapshots[i].id,
         )
+        archive_data.origin_visit_status_add([visit_status])
 
     all_visits = list(reversed(get_origin_visits(new_origin.to_dict())))
 
@@ -193,13 +214,26 @@ def test_api_lookup_origin_visit(
     archive_data.origin_add_one(new_origin)
     for i, visit_date in enumerate(visit_dates):
         origin_visit = archive_data.origin_visit_add(
-            new_origin.url, visit_date, type="git"
-        )
+            [
+                OriginVisit(
+                    origin=new_origin.url,
+                    date=visit_date,
+                    type="git",
+                    status="ongoing",
+                    snapshot=None,
+                )
+            ]
+        )[0]
         visit_id = origin_visit.visit
         archive_data.snapshot_add([new_snapshots[i]])
-        archive_data.origin_visit_update(
-            new_origin.url, visit_id, status="full", snapshot=new_snapshots[i].id
+        visit_status = OriginVisitStatus(
+            origin=new_origin.url,
+            visit=origin_visit.visit,
+            date=now(),
+            status="full",
+            snapshot=new_snapshots[i].id,
         )
+        archive_data.origin_visit_status_add([visit_status])
         url = reverse(
             "api-1-origin-visit",
             url_args={"origin_url": new_origin.url, "visit_id": visit_id},
@@ -244,14 +278,28 @@ def test_api_lookup_origin_visit_latest(
     visit_ids = []
     for i, visit_date in enumerate(visit_dates):
         origin_visit = archive_data.origin_visit_add(
-            new_origin.url, visit_date, type="git"
-        )
+            [
+                OriginVisit(
+                    origin=new_origin.url,
+                    date=visit_date,
+                    type="git",
+                    status="ongoing",
+                    snapshot=None,
+                )
+            ]
+        )[0]
         visit_ids.append(origin_visit.visit)
 
     archive_data.snapshot_add([new_snapshots[0]])
-    archive_data.origin_visit_update(
-        new_origin.url, visit_ids[0], status="full", snapshot=new_snapshots[0].id
+
+    visit_status = OriginVisitStatus(
+        origin=new_origin.url,
+        visit=visit_ids[0],
+        date=now(),
+        status="full",
+        snapshot=new_snapshots[0].id,
     )
+    archive_data.origin_visit_status_add([visit_status])
 
     url = reverse("api-1-origin-visit-latest", url_args={"origin_url": new_origin.url})
 
@@ -281,14 +329,28 @@ def test_api_lookup_origin_visit_latest_with_snapshot(
     visit_ids = []
     for i, visit_date in enumerate(visit_dates):
         origin_visit = archive_data.origin_visit_add(
-            new_origin.url, visit_date, type="git"
-        )
+            [
+                OriginVisit(
+                    origin=new_origin.url,
+                    date=visit_date,
+                    type="git",
+                    status="ongoing",
+                    snapshot=None,
+                )
+            ]
+        )[0]
         visit_ids.append(origin_visit.visit)
 
     archive_data.snapshot_add([new_snapshots[0]])
-    archive_data.origin_visit_update(
-        new_origin.url, visit_ids[0], status="full", snapshot=new_snapshots[0].id
+
+    visit_status = OriginVisitStatus(
+        origin=new_origin.url,
+        visit=visit_ids[0],
+        date=now(),
+        status="full",
+        snapshot=new_snapshots[0].id,
     )
+    archive_data.origin_visit_status_add([visit_status])
 
     url = reverse(
         "api-1-origin-visit-latest",
